package net.lemnik.eodsql.impl;

import java.lang.reflect.Array;
import java.util.Iterator;

/**
 * Created on 2009/05/23
 * @author Bernd Rinn
 */
enum BatchParameterKind { 
    ITERABLE, ARRAY, OTHER;
    
    static BatchParameterKind getKind(final Class<?> type) {
        if (type.isArray()) {
            return ARRAY;
        } else if (Iterable.class.isAssignableFrom(type)) {
            return ITERABLE;
        } else {
            return OTHER;
        }
    }
    
    Iterator<?> getIterator(final Object obj) {
        switch(this) {
            case ITERABLE:
                return ((Iterable<?>) obj).iterator();
            case ARRAY:
                return new Iterator()
                {
                    int i = 0;
                    public boolean hasNext()
                    {
                        return i < Array.getLength(obj);
                    }

                    public Object next()
                    {
                        return Array.get(obj, i++);
                    }

                    public void remove()
                    {
                        throw new UnsupportedOperationException();
                    }
                };
            default:
                return new Iterator()
                {
                    public boolean hasNext()
                    {
                        return true;
                    }

                    public Object next()
                    {
                        return obj;
                    }

                    public void remove()
                    {
                        throw new UnsupportedOperationException();
                    }
                };
        }
    }
}
