/*
 * Copyright 2009 ETH Zuerich, CISD
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package net.lemnik.eodsql.impl;

import java.sql.Connection;
import java.sql.SQLException;

import org.springframework.jdbc.support.SQLErrorCodeSQLExceptionTranslator;
import org.springframework.jdbc.support.SQLExceptionTranslator;

import net.lemnik.eodsql.impl.BaseQueryImpl.ConnectionSource;
import net.lemnik.eodsql.impl.BaseQueryImpl.DataSourceConnectionSource;

/**
 * A wrapper for Spring's <code>SQLExceptionTranslator</code>.
 * <p>
 * <i>Only construct this class when spring is in the class path!</i>
 * 
 * @author Bernd Rinn
 */
class SpringSQLExceptionTranslatorWrapper
{
    private final SQLExceptionTranslator exceptionTranslator;

    SpringSQLExceptionTranslatorWrapper(ConnectionSource source)
    {
        if (source instanceof DataSourceConnectionSource)
        {
            exceptionTranslator =
                    new SQLErrorCodeSQLExceptionTranslator(((DataSourceConnectionSource) source)
                            .getDataSource());
        } else
        {
            SQLExceptionTranslator exceptionTranslatorTmp = null;
            Connection conn = null;
            try
            {
                conn = source.getConnection();
                final String databaseProductName = conn.getMetaData().getDatabaseProductName();
                exceptionTranslatorTmp = new SQLErrorCodeSQLExceptionTranslator(databaseProductName);
            } catch (SQLException ex)
            {
                exceptionTranslatorTmp = new SQLErrorCodeSQLExceptionTranslator();
            } finally
            {
                if (conn != null)
                {
                    try
                    {
                        source.releaseConnection(conn);
                    } catch (SQLException ex1)
                    {
                        // Nothing we can do here.
                    }
                }
            }
            exceptionTranslator = exceptionTranslatorTmp;
        }
    }

    RuntimeException translate(String task, String sql, SQLException ex)
    {
        return exceptionTranslator.translate(task, sql, ex);
    }
}
